--[[
	DR TRACKER:
	This file handles the diminishing return tracking system. It uses combat log instead of the UNIT_AURA event, because the combat log fires for fading auras.
	- Diminishing Returns seem to have a 17 seconds duration AFTER the spell FADES.
	- The first DR has full duration, the second 50%, the last 25%.
	- A table needs to store the duration of DRs.
	- "COMBAT_LOG_EVENT_UNFILTERED" can track fading buffs/debuffs:
		SPELL_AURA_REMOVED -> Should fire when a buff/debuff has run out or was dispelled etc.
			-> After that 17 Sec duration on the DR-Type.
			-> GUID instead of unitID, because combat log doesn't announce unitIDs.
]]--
local diminishingReturn = 17;


function ArenaLive_DrTracker_Initialise (self, addonName, frameType, parent, numIcons)

	if ( not self ) then
		return;
	end
	
	if ( parent ) then
		parent.drTracker = self;
	end
	
	self.numIcons = numIcons;	
	self.drStorage = {};
	
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	
end

function ArenaLive_DrTracker_UpdateUnit (self, unit)
	self.unit = unit;
	self:SetScript("OnEvent", ArenaLive_DrTracker_OnEvent);
end

function ArenaLive_DrTracker_ResetFrame (self)
	self.unit = nil;
	self:SetScript("OnEvent", nil);
	self:Hide();
end

function ArenaLive_DrTracker_OnEvent(self, event, ...)
	
	if ( not self.unit or not UnitGUID(self.unit) ) then
		return;
	end
	
	if ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
		local event2 = select(2, ...);
		ArenaLive_DrTracker_Update(self, event2, ...);
	end

end

function ArenaLive_DrTracker_OnUpdate(self, elapsed)

	local drTypeCount = #(self.drStorage);
	
	if ( drTypeCount > 0 ) then
	
		for key, value in ipairs(self.drStorage) do
			if ( not self.drStorage[key]["waitForRemoval"] ) then
				-- The DR timer has started, so we need to count the duration down.
				local newDuration = self.drStorage[key]["duration"] - elapsed;
				
				if ( newDuration <= 0 ) then
					-- DR duration has run out, so we can delete the entry.
					table.remove(self.drStorage, key);
					ArenaLive_DrTracker_UpdateIcons(self);
				else
					self.drStorage[key]["duration"] = newDuration;
				end
			end
		
		end
	
	end

end

function ArenaLive_DrTracker_OnHide(self)
	local i = 1;
	
	while i <= self.numIcons do
		-- TODO: Icon reset
		i = i + 1
	end
end

function ArenaLive_DrTracker_Update(self, event, ...)
	
	local sourceGUID = select(4, ...);
	local affectedGUID = select(8, ...);
	local spellID = select(12, ...);
	
	--[[ Fix for Dominate Mind. Dominate mind would create a DR timer for the priest that casted dominate mind.
	Maybe we can remove the spellID later, but I need to test first, if it would interfere with spell reflected spells etc.]]--
	if ( sourceGUID == affectedGUID and spellID == 605 ) then
		return;
	end
	
	if ( event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH" ) then
	
		if ( ( affectedGUID == UnitGUID(self.unit) ) and ( ARENALIVE_SPELLS["diminishingReturns"][spellID] ) ) then
			self:Show();
			-- Check if the DRType we just received is already active.
			local match;
			for key, value in ipairs(self.drStorage) do
				if ( self.drStorage[key]["drType"] == ARENALIVE_SPELLS["diminishingReturns"][spellID] ) then
					match = key;
					break;
				end
			end
			
			if ( match ) then
				--[[ We already store this type of DR in our table. If the target isn't fully DR'd already,
				we must update the icon. Otherwise we just change the texture to make clear
				that the used spell is on the same DR. ]]--
				self.drStorage[match]["spellID"] = spellID;
				
				if ( self.drStorage[match]["stacks"] < 3 ) then
					self.drStorage[match]["waitForRemoval"] = true;
					self.drStorage[match]["startTime"] = nil;
					self.drStorage[match]["duration"] = diminishingReturn;
					self.drStorage[match]["stacks"] = self.drStorage[match]["stacks"] + 1;
					
					
					-- Update the Icon with the new data and reset cooldown, since we must wait until the aura is removed again.
					if ( match <= self.numIcons ) then
						ArenaLive_DrTrackerIcon_Update(self["icon"..match], self.drStorage[match]["spellID"], self.drStorage[match]["stacks"]);
						ArenaLive_DrTrackerIcon_SetDRTimer(self["icon"..match], GetTime(), 0);
					end	
				else
					ArenaLive_DrTrackerIcon_Update(self["icon"..match], self.drStorage[match]["spellID"], self.drStorage[match]["stacks"]);
				end
			else
				-- This is a new DR type, so we need to add it to our table.
				local numNewDR = #(self.drStorage) + 1;
				
				self.drStorage[numNewDR] = {};
				self.drStorage[numNewDR]["drType"] = ARENALIVE_SPELLS["diminishingReturns"][spellID];
				self.drStorage[numNewDR]["waitForRemoval"] = true;
				self.drStorage[numNewDR]["startTime"] = nil;
				self.drStorage[numNewDR]["spellID"] = spellID;
				self.drStorage[numNewDR]["stacks"] = 1;
				
				if ( numNewDR <= self.numIcons ) then
					ArenaLive_DrTrackerIcon_Update(self["icon"..numNewDR], spellID, 1)
				end
				
			end

		end	
	
	elseif ( event == "SPELL_AURA_REMOVED" ) then
	
		if ( ( affectedGUID == UnitGUID(self.unit) ) and ( ARENALIVE_SPELLS["diminishingReturns"][spellID] ) ) then
			-- A diminishing return aura has run out on our unit. Now the 15 sec CD must kick in.
			local match;
			for key, value in ipairs(self.drStorage) do
				if ( self.drStorage[key]["spellID"] == spellID ) then
					match = key;
					break;
				end
			end
			
			if ( match ) then
				
				if ( self.drStorage[match]["stacks"] < 3 or self.drStorage[match]["waitForRemoval"] ) then
					self.drStorage[match]["startTime"] = GetTime();
					self.drStorage[match]["waitForRemoval"] = false;
					self.drStorage[match]["duration"] = diminishingReturn;
					
					if ( match <= self.numIcons ) then
						ArenaLive_DrTrackerIcon_SetDRTimer(self["icon"..match], GetTime(), diminishingReturn);
					end
				end
			end
		end
	
	end

end

function ArenaLive_DrTracker_UpdateSize(drTracker, addonName, frameType)
	
	local numIcons = drTracker.numIcons
	local iconSize;
	
	if ( addonName ) then
		iconSize = ARENALIVE_DATABASE[addonName]["frameTypes"][frameType]["drTracker"]["iconSize"];
	else
		iconSize = ARENALIVE_DATABASE["frameTypes"][frameType]["drTracker"]["iconSize"];
	end
	
	local trackerWidth;
	if ( numIcons > 0 ) then
		trackerWidth = (iconSize * numIcons) + ( 5 * ( numIcons - 1 ) );
	else
		trackerWidth = 1;
	end
	
	drTracker:SetSize(trackerWidth, iconSize);
	
	for i = 1, numIcons do
		local icon = drTracker["icon"..i];
		ArenaLive_DrTrackerIcon_UpdateSize(icon, iconSize)
	end


end

function ArenaLive_DrTracker_UpdateIcons(drTracker)
	local drTypeCount = #(drTracker.drStorage);
	
	if ( drTypeCount > 0 and drTracker.numIcons > 0 ) then
		drTracker:Show();
		
		for i = 1, drTracker.numIcons do
			local frame = drTracker["icon"..i];
			
			if ( drTracker.drStorage[i] ) then
				ArenaLive_DrTrackerIcon_Update(frame, drTracker.drStorage[i]["spellID"], drTracker.drStorage[i]["stacks"]);
				
				-- If the timer for this frame has already been started, we need to set the cooldown correctly.
				if ( not drTracker.drStorage[i]["waitForRemoval"] and drTracker.drStorage[i]["startTime"] ) then
					ArenaLive_DrTrackerIcon_SetDRTimer(frame, drTracker.drStorage[i]["startTime"], diminishingReturn)
				end
			else
				ArenaLive_DrTrackerIcon_Reset(frame);
			end
		end
	else
		drTracker:Hide();
	end

end

function ArenaLive_DrTrackerIcon_Initialise(self, drTracker, texture, cooldown, drIndicator)
	local iconID = self:GetID();
	drTracker["icon"..iconID] = self;
	ArenaLive_Cooldown_Initialise(cooldown, self)
	
	self.texture = texture;
	self.drIndicator = drIndicator;

end

function ArenaLive_DrTrackerIcon_Update(icon, spellID, stacks)

	local _, _, texture = GetSpellInfo(spellID)
	
	icon:Show();
	icon.isActive = true;
	
	if ( texture ) then
		icon.texture:SetTexture(texture);
	end
	
	if ( stacks >= 3 ) then
		-- Red for full DR
		icon.drIndicator:SetTexture(1, 0, 0);
	elseif ( stacks == 2 ) then
		-- Yellow for 1/4 DR
		icon.drIndicator:SetTexture(1, 1, 0);
	elseif ( stacks == 1 ) then
		-- Green for 1/2 DR
		icon.drIndicator:SetTexture(0, 1, 0);
	end
end

function ArenaLive_DrTrackerIcon_UpdateSize(icon, newIconSize)

	icon:SetSize(newIconSize, newIconSize);

end

function ArenaLive_DrTrackerIcon_SetDRTimer(icon, startTime, duration)
	
	ArenaLive_Cooldown_Set(icon.cooldown, startTime, duration);

end

function ArenaLive_DrTrackerIcon_Reset(icon)
	
	icon:Hide();
	icon.isActive = nil;
	icon.texture:SetTexture();
	icon.drIndicator:SetTexture();
	icon.cooldown:SetCooldown(GetTime(), 0);
	

end